#pragma once

namespace Simulator {

	using namespace System;

	ref class MemorySystem;

	public ref class MachineState 
	{
	public:
		MachineState(String^ programRomPath, String^ microRomPath, unsigned long loadAddress, unsigned long bootAddress);

		void Reset();
		unsigned long GetMicroinstruction(unsigned long microAddress);
		unsigned long GetCurrentMicroaddress();
		unsigned long GetCurrentMicroinstruction();
		unsigned char Peek(unsigned long address); 
		void Store(unsigned long address, unsigned char value);

		String^ LCDText();

		unsigned long MicroStep();

		// 4-bit counter
		property unsigned char Phase;
		
		// 8-bit registers
		property unsigned char RegA;
		property unsigned char RegX;
		property unsigned char RegY;
		property unsigned char RegT;
		property unsigned char RegOpcode;
		
		// 24-bit registers
		property unsigned long RegPC;
		property unsigned long RegAR;
		property unsigned long RegSP;

		// flags
		property bool FlagNotCarry;
		property bool FlagNegative;
		property bool FlagNotZero;
		property bool FlagNotOverflow;
		property bool FlagNotAddressCarry;
		property bool FlagNotInterruptEnable;
		property bool FlagNotIrq;

		enum class AddressSelect
		{
			kSelectSP = 1,
			kSelectAR,
			kSelectPC
		};

		enum class Special
		{			
			kLoadIE = 0,
			kIncAR,
			kIncPC
		};

		enum class Dest
		{
			kLoadSPHI = 0,
			kLoadSPBank,
			kLoadARLO,
			kLoadARHI,
			kLoadARBank,
			kLoadPCLO,
			kLoadPCHI,
			kLoadPCBank,
			kLoadMem,
			kLoadSPLO,
			kLoadRegT,
			kLoadRegY,
			kLoadRegX,
			kLoadRegA
		};

		enum class ALULeft
		{
			kLeftZero,
			kLeftRegX,
			kLeftRegYorX7,
			kLeftRegA
		};

		enum class ALURight
		{
			kRightAddressBank,
			kRightAddressHi,
			kRightAddressLo,
			kRightRegT
		};

		enum class BitPos
		{
			kDest3 = 0,
			kAddressSelect = 2,
			kLoadOp = 3,
			kSPDirIEData = 4,
			kCountSP = 5,
			kSpecial = 7,
			kS1 = 8,
			kS0 = 9,
			kALUCarry = 10,
			kALUMode = 11,
			kALUOp = 15,
			kDriveALU = 16,
			kDest = 19,
			kALURight = 21,
			kALULeft = 23
		};

		static bool IsBitZero(unsigned long bitfield, BitPos pos) { return (bitfield & (1 << (unsigned long)pos)) == 0; }
		static bool IsBitOne(unsigned long bitfield, BitPos pos) { return (bitfield & (1 << (unsigned long)pos)) != 0; }
		static unsigned long FieldValue(unsigned long bitfield, BitPos pos, int len);

	protected:
		~MachineState() {}
		bool LoadMicrorom(String^ ronmPath, int romIndex);	

		MemorySystem^ m_memorySystem;
		
		static const int s_microromSize = 128 * 1024;
		array<unsigned long>^ m_microrom;

		unsigned long m_startAddress;

		unsigned char RunALU(unsigned long microOp, unsigned long effectiveAddress, bool% carryOut, bool% overflowOut);
	};
}

